#!/usr/bin/env python
# This code is Public Domain.
"""Python-Ogre Intermediate Tutorial 01: Animation, Walking Between Points, and Basic Quaternions."""
import ogre.renderer.OGRE as ogre

import SampleFramework as sf

class TutorialFrameListener(sf.FrameListener):
    """A FrameListener class that handles basic user input."""    
    
    def __init__(self, win, cam, sc, ent, walk , n ):
        # Subclass any Python-Ogre class and you must call its constructor.

        self.animationState = ent.getAnimationState('Idle')
        self.animationState.setLoop(True)
        self.animationState.setEnabled(True)

        # Set default values for variables
        self.walkSpeed = 35.0 # The speed at which the object is moving
        self.direction = ogre.Vector3().ZERO # The direction the object is moving
        self.distance = 0.0 # The distance the object has left to travel

        sf.FrameListener.__init__(self, win, cam)
        self.entity = ent # The entity we are animating
        self.node = sc # The SceneNode that the Entity is attached to
        self.walklist = walk # The list of points we are walking to
        self.counter = n # count no of the robot
        
    def nextLocation(self):

        #if len(self.walklist) == 0:
        #    return False
        if self.counter == 1:
            return False
        self.destination = self.walklist
        self.direction = self.destination - self.node.getPosition()
        self.distance = self.direction.normalise()
        
        src = self.node.getOrientation() * ogre.Vector3().UNIT_X
        if 1.0 + src.dotProduct(self.direction) < 0.0001:
            self.node.yaw(ogre.Degree(180))
        else:
            quat = src.getRotationTo(self.direction)
            self.node.rotate(quat)
        
        return True
    
    def frameStarted(self, evt):

        if self.direction == ogre.Vector3().ZERO:
            if self.nextLocation():
                # Set walking animation
                self.animationState = self.entity.getAnimationState('Walk')
                self.animationState.setLoop(True)
                self.animationState.setEnabled(True)
            else:
                self.animationState = self.entity.getAnimationState('Idle')
                self.animationState.setLoop(True)
                self.animationState.setEnabled(True)

        else:
            move = self.walkSpeed * evt.timeSinceLastFrame;
            self.distance -= move
            #print(self.distance)
            if self.distance <= 0.0:
                self.counter = 1
                self.node.setPosition(self.destination)
                self.direction = ogre.Vector3().ZERO
                if not self.nextLocation():
                    # Set Idle animation
                    self.animationState = self.entity.getAnimationState('Idle')
                    self.animationState.setLoop(True)
                    self.animationState.setEnabled(True)
            else:
                self.node.translate(self.direction * move)

        self.animationState.addTime(evt.timeSinceLastFrame)
        
        return sf.FrameListener.frameStarted(self, evt)
    
class TutorialApplication(sf.Application):
    """Application class."""
    
    def _createScene(self):
        sceneManager = self.sceneManager
        
        # Setup the ambient light.
        
        sceneManager.ambientLight = (1.0, 1.0, 1.0)
        
        # Create the entity
        
        self.entity = []
        self.node = []
        for x in xrange(2):
            x1 = str(x)
            x2 = 50*x
            name = "Robot"
            name_node = "RobotNode"
            ent_name = name + x1
            node_name = name_node + x1
            self.entity.append(sceneManager.createEntity (ent_name, "robot.mesh"))
            self.node.append(sceneManager.getRootSceneNode().createChildSceneNode (node_name, ( 0 , 0 , x2 ) ))



        for x in xrange(2):
            self.node[x].attachObject (self.entity[x])

        # Create the walking list
        
        self.walklist = []

        for x in xrange(2):

            x3 = 50 * x;
            self.walklist.append(ogre.Vector3(100, 0, x3))
        
        # Add some objects so we can see movement
        
        knotEnt = sceneManager.createEntity('Knot1', 'knot.mesh')
        knotNode = sceneManager.getRootSceneNode().createChildSceneNode('Knot1Node', (0, 0, -100))
        knotNode.attachObject(knotEnt)
        knotNode.scale = (.1, .1, .1)
        knotEnt = sceneManager.createEntity('Knot2', 'knot.mesh')
        knotNode = sceneManager.getRootSceneNode().createChildSceneNode('Knot2Node', (0, 0, 150))
        knotNode.attachObject(knotEnt)
        knotNode.scale = (.1, .1, .1)
        
        #**********************END************************#            
        
    def _createCamera(self):
        self.camera = self.sceneManager.createCamera('PlayerCam')
        
        # Set the camera to look at our handiwork
        
        self.camera.position = (0,0,0)
        self.camera.pitch(ogre.Degree(-30))
        self.camera.yaw(ogre.Degree(-15))

        #**********************END************************#
        
    def _createFrameListener(self):


        for x in range(2):
            counter = 0
            self.frameListener = TutorialFrameListener(self.renderWindow,
                                                        self.camera,
                                                        self.node[x],
                                                        self.entity[x],
                                                        self.walklist[x],
                                                        counter
                                                        )
            print x
            print(self.node[x])
            print(self.entity[x])
            print(self.walklist[x])
            print(self.node[x].getPosition())
            
            self.root.addFrameListener(self.frameListener)
            self.frameListener.showDebugOverlay(True)
            
        
if __name__ == '__main__':
    ta = TutorialApplication()
    ta.go()
